@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -24,6 +24,7 @@
 
 #include <connman/types.h>
 #include <connman/device.h>
+#include <connman/ipconfig.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -37,19 +38,24 @@ extern "C" {
 
 enum connman_network_type {
 	CONNMAN_NETWORK_TYPE_UNKNOWN       = 0,
-	CONNMAN_NETWORK_TYPE_WIFI          = 1,
-	CONNMAN_NETWORK_TYPE_WIMAX         = 2,
+	CONNMAN_NETWORK_TYPE_ETHERNET      = 1,
+	CONNMAN_NETWORK_TYPE_WIFI          = 2,
+	CONNMAN_NETWORK_TYPE_WIMAX         = 3,
 	CONNMAN_NETWORK_TYPE_BLUETOOTH_PAN = 8,
 	CONNMAN_NETWORK_TYPE_BLUETOOTH_DUN = 9,
-	CONNMAN_NETWORK_TYPE_MBM           = 22,
-	CONNMAN_NETWORK_TYPE_HSO           = 23,
+	CONNMAN_NETWORK_TYPE_CELLULAR      = 10,
 	CONNMAN_NETWORK_TYPE_VENDOR        = 10000,
 };
 
 enum connman_network_protocol {
 	CONNMAN_NETWORK_PROTOCOL_UNKNOWN = 0,
 	CONNMAN_NETWORK_PROTOCOL_IP      = 1,
-	CONNMAN_NETWORK_PROTOCOL_PPP     = 2,
+};
+
+enum connman_network_error {
+	CONNMAN_NETWORK_ERROR_UNKNOWN         = 0,
+	CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL  = 1,
+	CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL  = 2,
 };
 
 struct connman_network;
@@ -63,6 +69,9 @@ enum connman_network_type connman_network_get_type(struct connman_network *netwo
 const char *connman_network_get_identifier(struct connman_network *network);
 
 const char *connman_network_get_path(struct connman_network *network);
+
+struct connman_element *connman_network_get_element(
+				struct connman_network *network);
 void connman_network_set_index(struct connman_network *network, int index);
 int connman_network_get_index(struct connman_network *network);
 
@@ -77,23 +86,34 @@ int connman_network_set_available(struct connman_network *network,
 connman_bool_t connman_network_get_available(struct connman_network *network);
 int connman_network_set_associating(struct connman_network *network,
 						connman_bool_t associating);
+void connman_network_set_error(struct connman_network *network,
+					enum connman_network_error error);
 int connman_network_set_connected(struct connman_network *network,
 						connman_bool_t connected);
 connman_bool_t connman_network_get_connected(struct connman_network *network);
 
 connman_bool_t connman_network_get_associating(struct connman_network *network);
 
+void connman_network_set_method(struct connman_network *network,
+					enum connman_ipconfig_method method);
+
 int connman_network_set_address(struct connman_network *network,
 				const void *address, unsigned int size);
 int connman_network_set_name(struct connman_network *network,
 							const char *name);
 int connman_network_set_strength(struct connman_network *network,
 						connman_uint8_t strength);
+int connman_network_set_roaming(struct connman_network *network,
+						connman_bool_t roaming);
 
 int connman_network_set_string(struct connman_network *network,
 					const char *key, const char *value);
 const char *connman_network_get_string(struct connman_network *network,
 							const char *key);
+int connman_network_set_bool(struct connman_network *network,
+					const char *key, connman_bool_t value);
+connman_bool_t connman_network_get_bool(struct connman_network *network,
+							const char *key);
 int connman_network_set_uint8(struct connman_network *network,
 					const char *key, connman_uint8_t value);
 connman_uint8_t connman_network_get_uint8(struct connman_network *network,
@@ -120,6 +140,7 @@ struct connman_network_driver {
 	void (*remove) (struct connman_network *network);
 	int (*connect) (struct connman_network *network);
 	int (*disconnect) (struct connman_network *network);
+	int (*setup) (struct connman_network *network, const char *key);
 };
 
 int connman_network_driver_register(struct connman_network_driver *driver);
