@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -32,6 +32,7 @@
 
 #define CONNMAN_API_SUBJECT_TO_CHANGE
 #include <connman/plugin.h>
+#include <connman/utsname.h>
 #include <connman/dhcp.h>
 #include <connman/task.h>
 #include <connman/log.h>
@@ -87,6 +88,8 @@ static void dhclient_notify(struct connman_task *task,
 			connman_dhcp_set_value(dhcp, "Timeserver", value);
 		} else if (g_ascii_strcasecmp(key, "new_interface_mtu") == 0) {
 			connman_dhcp_set_value(dhcp, "MTU", value);
+		} else if (g_ascii_strcasecmp(key, "new_proxy_auto_config") == 0) {
+			connman_dhcp_set_value(dhcp, "PAC", value);
 		}
 
 		dbus_message_iter_next(&dict);
@@ -127,6 +130,7 @@ static void dhclient_died(struct connman_task *task, void *user_data)
 static void dhclient_setup(struct connman_task *task, const char *ifname)
 {
 	const char *path, *intf = "org.moblin.connman.Task";
+	const char *hostname;
 
 	path = connman_task_get_path(task);
 
@@ -141,6 +145,13 @@ static void dhclient_setup(struct connman_task *task, const char *ifname)
 							STATEDIR, ifname);
 	connman_task_add_argument(task, "-cf", "%s/dhclient.conf", SCRIPTDIR);
 	connman_task_add_argument(task, "-sf", "%s/dhclient-script", SCRIPTDIR);
+
+	hostname = connman_utsname_get_hostname();
+#ifdef HAVE_DHCLIENT_HOSTNAME
+	if (hostname != NULL)
+		connman_task_add_argument(task, "-H", hostname);
+#endif
+
 	connman_task_add_argument(task, ifname, NULL);
 	connman_task_add_argument(task, "-n", NULL);
 }
