@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -30,6 +30,7 @@
 
 #define CONNMAN_API_SUBJECT_TO_CHANGE
 #include <connman/plugin.h>
+#include <connman/element.h>
 #include <connman/device.h>
 #include <connman/network.h>
 #include <connman/dbus.h>
@@ -43,13 +44,18 @@
 #define OFONO_GPRS_INTERFACE		OFONO_SERVICE ".DataConnectionManager"
 #define OFONO_SIM_INTERFACE		OFONO_SERVICE ".SimManager"
 #define OFONO_PRI_CONTEXT_INTERFACE	OFONO_SERVICE ".PrimaryDataContext"
+#define OFONO_REGISTRATION_INTERFACE	OFONO_SERVICE ".NetworkRegistration"
 
 #define PROPERTY_CHANGED		"PropertyChanged"
 #define GET_PROPERTIES			"GetProperties"
 #define SET_PROPERTY			"SetProperty"
+#define CREATE_CONTEXT			"CreateContext"
 
 #define TIMEOUT 5000
 
+#define CONTEXT_NAME "3G Connection"
+#define CONTEXT_TYPE "internet"
+
 static DBusConnection *connection;
 
 static GHashTable *modem_hash = NULL;
@@ -89,6 +95,8 @@ static void powered_reply(DBusPendingCall *call, void *user_data)
 	}
 
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static int gprs_change_powered(const char *path, dbus_bool_t powered)
@@ -110,7 +118,7 @@ static int gprs_change_powered(const char *path, dbus_bool_t powered)
 	dbus_message_set_auto_start(message, FALSE);
 
 	dbus_message_iter_init_append(message, &iter);
-	connman_dbus_property_append_variant(&iter, "Powered",
+	connman_dbus_property_append_basic(&iter, "Powered",
 						DBUS_TYPE_BOOLEAN, &powered);
 
 	if (dbus_connection_send_with_reply(connection, message,
@@ -177,18 +185,31 @@ static char *get_ident(const char *path)
 	return ident;
 }
 
-static void config_network_reply(DBusPendingCall *call, void *user_data)
+static void create_service(struct connman_network *network)
+{
+	const char *path;
+	char *group;
+
+	DBG("");
+
+	path = connman_network_get_string(network, "Path");
+
+	group = get_ident(path);
+
+	connman_network_set_group(network, group);
+
+	g_free(group);
+}
+
+static void set_network_name_reply(DBusPendingCall *call, void *user_data)
 {
 	struct connman_network *network = user_data;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
-	gboolean internet_type = FALSE;
 
 	DBG("network %p", network);
 
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		goto done;
 
 	if (dbus_message_iter_init(reply, &array) == FALSE)
 		goto done;
@@ -208,12 +229,97 @@ static void config_network_reply(DBusPendingCall *call, void *user_data)
 		dbus_message_iter_next(&entry);
 		dbus_message_iter_recurse(&entry, &value);
 
-		if (g_str_equal(key, "Name") == TRUE) {
+		/*
+		 * 'Operator' is deprecated since version 0.20, but
+		 * keep it here for backward compatibility reasons.
+		 */
+		if (g_str_equal(key, "Operator") == TRUE ||
+				g_str_equal(key, "Name") == TRUE) {
 			const char *name;
 
 			dbus_message_iter_get_basic(&value, &name);
 			connman_network_set_name(network, name);
-		} else if (g_str_equal(key, "Type") == TRUE) {
+			create_service(network);
+		}
+
+		dbus_message_iter_next(&dict);
+	}
+done:
+	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
+}
+
+static void set_network_name(struct connman_network *network)
+{
+	struct connman_device *device;
+	DBusMessage *message;
+	DBusPendingCall *call;
+	const char *path;
+
+	device = connman_network_get_device(network);
+
+	path = connman_device_get_string(device, "Path");
+	if (path == NULL)
+		return;
+
+	DBG("path %s", path);
+
+	message = dbus_message_new_method_call(OFONO_SERVICE, path,
+				OFONO_REGISTRATION_INTERFACE, GET_PROPERTIES);
+	if (message == NULL)
+		return;
+
+	dbus_message_set_auto_start(message, FALSE);
+
+	if (dbus_connection_send_with_reply(connection, message,
+						&call, TIMEOUT) == FALSE) {
+		connman_error("Failed to get operator");
+		goto done;
+	}
+
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		goto done;
+	}
+
+	dbus_pending_call_set_notify(call, set_network_name_reply,
+						(void *)network, NULL);
+
+done:
+	dbus_message_unref(message);
+}
+
+static void config_network_reply(DBusPendingCall *call, void *user_data)
+{
+	struct connman_network *network = user_data;
+	DBusMessage *reply;
+	DBusMessageIter array, dict;
+	gboolean internet_type = FALSE;
+
+	DBG("network %p", network);
+
+	reply = dbus_pending_call_steal_reply(call);
+
+	if (dbus_message_iter_init(reply, &array) == FALSE)
+		goto done;
+
+	if (dbus_message_iter_get_arg_type(&array) != DBUS_TYPE_ARRAY)
+		goto done;
+
+	dbus_message_iter_recurse(&array, &dict);
+
+	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
+		DBusMessageIter entry, value;
+		const char *key;
+
+		dbus_message_iter_recurse(&dict, &entry);
+		dbus_message_iter_get_basic(&entry, &key);
+
+		dbus_message_iter_next(&entry);
+		dbus_message_iter_recurse(&entry, &value);
+
+		if (g_str_equal(key, "Type") == TRUE) {
 			const char *type;
 
 			dbus_message_iter_get_basic(&value, &type);
@@ -222,6 +328,7 @@ static void config_network_reply(DBusPendingCall *call, void *user_data)
 
 				connman_network_set_protocol(network,
 						CONNMAN_NETWORK_PROTOCOL_IP);
+				set_network_name(network);
 			} else {
 				internet_type = FALSE;
 
@@ -233,21 +340,10 @@ static void config_network_reply(DBusPendingCall *call, void *user_data)
 		dbus_message_iter_next(&dict);
 	}
 
-	if (internet_type == TRUE) {
-		const char *path;
-		char *group;
-
-		path = connman_network_get_string(network, "Path");
-
-		group = get_ident(path);
-
-		connman_network_set_group(network, group);
-
-		g_free(group);
-	}
-
 done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static void config_network(struct connman_network *network, const char *path)
@@ -307,8 +403,10 @@ static gboolean pending_network_is_available(
 
 	/* Modem may be removed during waiting for active reply */
 	device  = connman_network_get_device(pending_network);
-	if (device == NULL)
+	if (device == NULL) {
+		DBG("Modem is removed");
 		return FALSE;
+	}
 
 	identifier = connman_network_get_identifier(pending_network);
 
@@ -335,12 +433,10 @@ static void set_active_reply(DBusPendingCall *call, void *user_data)
 
 	DBG("network %p", network);
 
-	if (pending_network_is_available(network) == FALSE)
-		return;
-
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		return;
+
+	if (pending_network_is_available(network) == FALSE)
+		goto done;
 
 	dbus_error_init(&error);
 	if (dbus_set_error_from_message(&error, reply)) {
@@ -356,7 +452,10 @@ static void set_active_reply(DBusPendingCall *call, void *user_data)
 	} else
 		pending_network = network;
 
+done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static int set_network_active(struct connman_network *network,
@@ -381,7 +480,7 @@ static int set_network_active(struct connman_network *network,
 	dbus_message_set_auto_start(message, FALSE);
 
 	dbus_message_iter_init_append(message, &iter);
-	connman_dbus_property_append_variant(&iter, "Active",
+	connman_dbus_property_append_basic(&iter, "Active",
 						DBUS_TYPE_BOOLEAN, &active);
 
 	if (dbus_connection_send_with_reply(connection, message,
@@ -409,6 +508,70 @@ static int set_network_active(struct connman_network *network,
 	return 0;
 }
 
+static void set_apn_reply(DBusPendingCall *call, void *user_data)
+{
+	DBusMessage *reply;
+	DBusError error;
+
+	reply = dbus_pending_call_steal_reply(call);
+
+	dbus_error_init(&error);
+	if (dbus_set_error_from_message(&error, reply)) {
+		connman_error("%s", error.message);
+
+		dbus_error_free(&error);
+	}
+
+	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
+}
+
+static void set_apn(struct connman_network *network)
+{
+	DBusMessage *message;
+	DBusPendingCall *call;
+	DBusMessageIter iter;
+	const char *apn, *path;
+
+	apn = connman_network_get_string(network, "Cellular.APN");
+	if (apn == NULL)
+		return;
+
+	path = connman_network_get_string(network, "Path");
+	if (path == NULL)
+		return;
+
+	DBG("path %s, apn %s", path, apn);
+
+	message = dbus_message_new_method_call(OFONO_SERVICE, path,
+				OFONO_PRI_CONTEXT_INTERFACE, SET_PROPERTY);
+	if (message == NULL)
+		return;
+
+	dbus_message_set_auto_start(message, FALSE);
+
+	dbus_message_iter_init_append(message, &iter);
+	connman_dbus_property_append_basic(&iter, "AccessPointName",
+						DBUS_TYPE_STRING, &apn);
+
+	if (dbus_connection_send_with_reply(connection, message,
+					&call, TIMEOUT) == FALSE) {
+		dbus_message_unref(message);
+		return;
+	}
+
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return;
+	}
+
+	dbus_pending_call_set_notify(call, set_apn_reply, NULL, NULL);
+
+	dbus_message_unref(message);
+}
+
 static int network_connect(struct connman_network *network)
 {
 	if (connman_network_get_index(network) >= 0)
@@ -430,6 +593,16 @@ static void network_remove(struct connman_network *network)
 	DBG("network %p", network);
 }
 
+static int network_setup(struct connman_network *network, const char *key)
+{
+	DBG("");
+
+	if (g_strcmp0(key, "Cellular.APN") == 0)
+		set_apn(network);
+
+	return 0;
+}
+
 static struct connman_network_driver network_driver = {
 	.name		= "network",
 	.type		= CONNMAN_NETWORK_TYPE_CELLULAR,
@@ -437,12 +610,15 @@ static struct connman_network_driver network_driver = {
 	.remove		= network_remove,
 	.connect	= network_connect,
 	.disconnect	= network_disconnect,
+	.setup		= network_setup,
 };
 
 static void add_network(struct connman_device *device, const char *path)
 {
 	struct connman_network *network;
 	char *ident;
+	const char *mcc;
+	const char *mnc;
 
 	DBG("device %p path %s", device, path);
 
@@ -462,6 +638,15 @@ static void add_network(struct connman_device *device, const char *path)
 	connman_network_set_string(network, "Path", path);
 	connman_network_set_available(network, TRUE);
 	connman_network_set_index(network, -1);
+
+	mcc = connman_device_get_string(device, "MCC");
+	if (mcc != NULL)
+		connman_network_set_string(network, "Cellular.MCC", mcc);
+
+	mnc = connman_device_get_string(device, "MNC");
+	if (mnc != NULL)
+		connman_network_set_string(network, "Cellular.MNC", mnc);
+
 	connman_device_add_network(device, network);
 }
 
@@ -485,6 +670,75 @@ static void add_networks(struct connman_device *device, DBusMessageIter *array)
 	}
 }
 
+static void create_context_reply(DBusPendingCall *call, void *user_data)
+{
+	DBusMessage *reply;
+	DBusError error;
+
+	DBG("");
+
+	dbus_error_init(&error);
+
+	reply = dbus_pending_call_steal_reply(call);
+
+	if (dbus_set_error_from_message(&error, reply)) {
+		connman_error("%s", error.message);
+		dbus_error_free(&error);
+	}
+
+	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
+}
+
+static void add_default_context(DBusMessageIter *array,
+		const char *path, const char *name, const char *type)
+{
+	DBusMessageIter entry;
+	DBusMessage *message;
+	DBusPendingCall *call;
+
+	if (path == NULL)
+		return;
+
+	DBG("");
+
+	dbus_message_iter_recurse(array, &entry);
+
+	if (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_OBJECT_PATH)
+		return;
+
+	DBG("path %s, name %s, type %s", path, name, type);
+
+	message = dbus_message_new_method_call(OFONO_SERVICE, path,
+					OFONO_GPRS_INTERFACE, CREATE_CONTEXT);
+	if (message == NULL)
+		return;
+
+	dbus_message_set_auto_start(message, FALSE);
+
+	dbus_message_append_args(message, DBUS_TYPE_STRING,
+					&name, DBUS_TYPE_STRING,
+						&type, DBUS_TYPE_INVALID);
+
+	if (dbus_connection_send_with_reply(connection, message,
+						&call, TIMEOUT) == FALSE) {
+		connman_error("Failed to create default context");
+		dbus_message_unref(message);
+		return;
+	}
+
+	if (call == NULL) {
+		connman_error("D-Bus connection not available");
+		dbus_message_unref(message);
+		return;
+	}
+
+	dbus_pending_call_set_notify(call, create_context_reply, NULL, NULL);
+
+	dbus_message_unref(message);
+}
+
 static void check_networks_reply(DBusPendingCall *call, void *user_data)
 {
 	struct connman_device *device = user_data;
@@ -495,8 +749,6 @@ static void check_networks_reply(DBusPendingCall *call, void *user_data)
 	DBG("device %p", device);
 
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		goto done;
 
 	if (dbus_message_iter_init(reply, &array) == FALSE)
 		goto done;
@@ -522,12 +774,12 @@ static void check_networks_reply(DBusPendingCall *call, void *user_data)
 			dbus_message_iter_get_basic(&value, &attached);
 			DBG("Attached %d", attached);
 		} else if (g_str_equal(key, "PrimaryContexts") == TRUE) {
-			contexts = value;
-		} else if (g_str_equal(key, "Status") == TRUE) {
-			const char *status;
+			const char *path;
 
-			dbus_message_iter_get_basic(&value, &status);
-			/* FIXME: add roaming support */
+			path = connman_device_get_string(device, "Path");
+			contexts = value;
+			add_default_context(&contexts, path,
+					CONTEXT_NAME, CONTEXT_TYPE);
 		} else if (g_str_equal(key, "Powered") == TRUE) {
 			dbus_bool_t powered;
 
@@ -544,6 +796,8 @@ static void check_networks_reply(DBusPendingCall *call, void *user_data)
 
 done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static void check_networks(struct modem_data *modem)
@@ -586,7 +840,8 @@ done:
 	dbus_message_unref(message);
 }
 
-static void add_device(const char *path, const char *imsi)
+static void add_device(const char *path, const char *imsi,
+				const char *mcc, const char *mnc)
 {
 	struct modem_data *modem;
 	struct connman_device *device;
@@ -612,6 +867,10 @@ static void add_device(const char *path, const char *imsi)
 	connman_device_set_mode(device, CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE);
 
 	connman_device_set_string(device, "Path", path);
+	if (mcc != NULL)
+		connman_device_set_string(device, "MCC", mcc);
+	if (mnc != NULL)
+		connman_device_set_string(device, "MNC", mnc);
 
 	if (connman_device_register(device) < 0) {
 		connman_device_unref(device);
@@ -626,14 +885,17 @@ static void add_device(const char *path, const char *imsi)
 static void sim_properties_reply(DBusPendingCall *call, void *user_data)
 {
 	const char *path = user_data;
+	const char *imsi;
+	char *mcc = NULL;
+	char *mnc = NULL;
+	/* If MobileNetworkCodeLength is not provided, mnc_length is 0 */
+	unsigned char mnc_length = 0;
 	DBusMessage *reply;
 	DBusMessageIter array, dict;
 
 	DBG("path %s", path);
 
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		return;
 
 	if (dbus_message_iter_init(reply, &array) == FALSE)
 		goto done;
@@ -645,7 +907,7 @@ static void sim_properties_reply(DBusPendingCall *call, void *user_data)
 
 	while (dbus_message_iter_get_arg_type(&dict) == DBUS_TYPE_DICT_ENTRY) {
 		DBusMessageIter entry, value;
-		const char *key, *imsi;
+		const char *key;
 
 		dbus_message_iter_recurse(&dict, &entry);
 		dbus_message_iter_get_basic(&entry, &key);
@@ -653,17 +915,41 @@ static void sim_properties_reply(DBusPendingCall *call, void *user_data)
 		dbus_message_iter_next(&entry);
 		dbus_message_iter_recurse(&entry, &value);
 
-		if (g_str_equal(key, "SubscriberIdentity") == TRUE) {
+		if (g_str_equal(key, "SubscriberIdentity") == TRUE)
 			dbus_message_iter_get_basic(&value, &imsi);
-
-			add_device(path, imsi);
-		}
+		/*
+		 * 'MobileNetworkCodeLength' is deprecated since version 0.20, but
+		 * keep it here for backward compatibility reasons.
+		 */
+		else if (g_str_equal(key, "MobileNetworkCodeLength") == TRUE)
+			dbus_message_iter_get_basic(&value,
+						(void *) &mnc_length);
+		else if (g_str_equal(key, "MobileCountryCode") == TRUE)
+			dbus_message_iter_get_basic(&value,
+						(void *) &mcc);
+		else if (g_str_equal(key, "MobileNetworkCode") == TRUE)
+			dbus_message_iter_get_basic(&value,
+						(void *) &mnc);
 
 		dbus_message_iter_next(&dict);
 	}
 
+	if (mnc_length == 2 || mnc_length == 3) {
+		mcc = g_strndup(imsi, 3);
+		mnc = g_strndup(imsi + 3, mnc_length);
+	}
+
+	add_device(path, imsi, mcc, mnc);
+
+	if (mnc_length == 2 || mnc_length == 3) {
+		g_free(mcc);
+		g_free(mnc);
+	}
+
 done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static void get_imsi(const char *path)
@@ -717,7 +1003,7 @@ static int modem_change_powered(const char *path, dbus_bool_t powered)
 	dbus_message_set_auto_start(message, FALSE);
 
 	dbus_message_iter_init_append(message, &iter);
-	connman_dbus_property_append_variant(&iter, "Powered",
+	connman_dbus_property_append_basic(&iter, "Powered",
 						DBUS_TYPE_BOOLEAN, &powered);
 
 	if (dbus_connection_send_with_reply(connection, message,
@@ -796,8 +1082,6 @@ static void modem_properties_reply(DBusPendingCall *call, void *user_data)
 	DBG("path %s", path);
 
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		goto done;
 
 	if (dbus_message_iter_init(reply, &array) == FALSE)
 		goto done;
@@ -825,7 +1109,7 @@ static void modem_properties_reply(DBusPendingCall *call, void *user_data)
 				modem_change_powered(path, TRUE);
 				break;
 			}
-		} else if (g_str_equal(key, "Interface") == TRUE) {
+		} else if (g_str_equal(key, "Interfaces") == TRUE) {
 			if (modem_has_gprs(&value) == TRUE)
 				get_imsi(path);
 		}
@@ -835,6 +1119,8 @@ static void modem_properties_reply(DBusPendingCall *call, void *user_data)
 
 done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static void get_modem_properties(struct modem_data *modem)
@@ -930,8 +1216,6 @@ static void manager_properties_reply(DBusPendingCall *call, void *user_data)
 	DBG("");
 
 	reply = dbus_pending_call_steal_reply(call);
-	if (reply == NULL)
-		goto done;
 
 	if (dbus_message_iter_init(reply, &array) == FALSE)
 		goto done;
@@ -961,6 +1245,8 @@ static void manager_properties_reply(DBusPendingCall *call, void *user_data)
 
 done:
 	dbus_message_unref(reply);
+
+	dbus_pending_call_unref(call);
 }
 
 static void modem_remove_device(struct modem_data *modem)
@@ -1033,7 +1319,8 @@ static void ofono_disconnect(DBusConnection *connection, void *user_data)
 	modem_hash = NULL;
 }
 
-static void modem_changed(DBusConnection *connection, DBusMessage *message)
+static gboolean modem_changed(DBusConnection *connection, DBusMessage *message,
+				void *user_data)
 {
 	const char *path = dbus_message_get_path(message);
 	struct modem_data *modem;
@@ -1044,10 +1331,10 @@ static void modem_changed(DBusConnection *connection, DBusMessage *message)
 
 	modem = g_hash_table_lookup(modem_hash, path);
 	if (modem == NULL)
-		return;
+		return TRUE;
 
 	if (dbus_message_iter_init(message, &iter) == FALSE)
-		return;
+		return TRUE;
 
 	dbus_message_iter_get_basic(&iter, &key);
 
@@ -1059,7 +1346,7 @@ static void modem_changed(DBusConnection *connection, DBusMessage *message)
 
 		dbus_message_iter_get_basic(&value, &powered);
 		if (powered == TRUE)
-			return;
+			return TRUE;
 
 		modem_remove_device(modem);
 	} else if (g_str_equal(key, "Interfaces") == TRUE) {
@@ -1069,9 +1356,12 @@ static void modem_changed(DBusConnection *connection, DBusMessage *message)
 		} else if (modem->device != NULL)
 			modem_remove_device(modem);
 	}
+
+	return TRUE;
 }
 
-static void gprs_changed(DBusConnection *connection, DBusMessage *message)
+static gboolean gprs_changed(DBusConnection *connection, DBusMessage *message,
+				void *user_data)
 {
 	const char *path = dbus_message_get_path(message);
 	struct modem_data *modem;
@@ -1082,10 +1372,10 @@ static void gprs_changed(DBusConnection *connection, DBusMessage *message)
 
 	modem = g_hash_table_lookup(modem_hash, path);
 	if (modem == NULL)
-		return;
+		return TRUE;
 
 	if (dbus_message_iter_init(message, &iter) == FALSE)
-		return;
+		return TRUE;
 
 	dbus_message_iter_get_basic(&iter, &key);
 
@@ -1104,27 +1394,23 @@ static void gprs_changed(DBusConnection *connection, DBusMessage *message)
 		else if (modem->device != NULL)
 			connman_device_remove_all_networks(modem->device);
 
-	} else if (g_str_equal(key, "Status") == TRUE) {
-		const char *status;
-		dbus_message_iter_get_basic(&value, &status);
-
-		DBG("status %s", status);
-
-		/* FIXME: add roaming support */
 	} else if (g_str_equal(key, "PrimaryContexts") == TRUE) {
 		check_networks(modem);
 	} else if (g_str_equal(key, "Powered") == TRUE) {
 		dbus_bool_t powered;
 
 		if (modem->device == NULL)
-			return;
+			return TRUE;
 
 		dbus_message_iter_get_basic(&value, &powered);
 		connman_device_set_powered(modem->device, powered);
 	}
+
+	return TRUE;
 }
 
-static void manager_changed(DBusConnection *connection, DBusMessage *message)
+static gboolean manager_changed(DBusConnection *connection,
+				DBusMessage *message, void *user_data)
 {
 	const char *path = dbus_message_get_path(message);
 	DBusMessageIter iter, value;
@@ -1133,7 +1419,7 @@ static void manager_changed(DBusConnection *connection, DBusMessage *message)
 	DBG("path %s", path);
 
 	if (dbus_message_iter_init(message, &iter) == FALSE)
-		return;
+		return TRUE;
 
 	dbus_message_iter_get_basic(&iter, &key);
 
@@ -1142,9 +1428,45 @@ static void manager_changed(DBusConnection *connection, DBusMessage *message)
 
 	if (g_str_equal(key, "Modems") == TRUE)
 		update_modems(&value);
+
+	return TRUE;
 }
 
-static void update_settings(DBusMessageIter *array)
+static void get_dns(DBusMessageIter *array, struct connman_element *parent)
+{
+	DBusMessageIter entry;
+	gchar *nameserver = NULL, *nameserver_old = NULL;
+
+	DBG("");
+
+	dbus_message_iter_recurse(array, &entry);
+
+	while (dbus_message_iter_get_arg_type(&entry) == DBUS_TYPE_STRING) {
+		const char *dns;
+
+		dbus_message_iter_get_basic(&entry, &dns);
+
+		DBG("dns %s", dns);
+
+		if (nameserver == NULL) {
+
+			nameserver = g_strdup(dns);
+		} else {
+
+			nameserver_old = nameserver;
+			nameserver = g_strdup_printf("%s %s",
+						nameserver_old, dns);
+			g_free(nameserver_old);
+		}
+
+		dbus_message_iter_next(&entry);
+	}
+
+	parent->ipv4.nameserver = nameserver;
+}
+
+static void update_settings(DBusMessageIter *array,
+			struct connman_element *parent)
 {
 	DBusMessageIter dict;
 	const char *interface = NULL;
@@ -1187,19 +1509,44 @@ static void update_settings(DBusMessageIter *array)
 
 			dbus_message_iter_get_basic(&value, &method);
 			if (g_strcmp0(method, "static") == 0) {
-				DBG("static");
+
+				parent->ipv4.method =
+					CONNMAN_IPCONFIG_METHOD_FIXED;
 			} else if (g_strcmp0(method, "dhcp") == 0) {
-				DBG("dhcp");
+
+				parent->ipv4.method =
+					CONNMAN_IPCONFIG_METHOD_DHCP;
 				break;
 			}
-		} else if (g_str_equal(key, "address") == TRUE) {
+		} else if (g_str_equal(key, "Address") == TRUE) {
 			const char *address;
 
 			dbus_message_iter_get_basic(&value, &address);
 
 			DBG("address %s", address);
+
+			parent->ipv4.address = g_strdup(address);
+		} else if (g_str_equal(key, "Netmask") == TRUE) {
+			const char *netmask;
+
+			dbus_message_iter_get_basic(&value, &netmask);
+
+			DBG("netmask %s", netmask);
+
+			parent->ipv4.netmask = g_strdup(netmask);
+		} else if (g_str_equal(key, "DomainNameServers") == TRUE) {
+
+			get_dns(&value, parent);
+		} else if (g_str_equal(key, "Gateway") == TRUE) {
+			const char *gateway;
+
+			dbus_message_iter_get_basic(&value, &gateway);
+
+			DBG("gateway %s", gateway);
+
+			parent->ipv4.gateway = g_strdup(gateway);
 		}
-		/* FIXME: add static setting */
+
 		dbus_message_iter_next(&dict);
 	}
 
@@ -1208,10 +1555,71 @@ static void update_settings(DBusMessageIter *array)
 		connman_network_set_index(pending_network, -1);
 }
 
-static void pri_context_changed(DBusConnection *connection,
-					DBusMessage *message)
+static void cleanup_ipconfig(struct connman_element *parent)
+{
+	g_free(parent->ipv4.address);
+	parent->ipv4.address = NULL;
+
+	g_free(parent->ipv4.netmask);
+	parent->ipv4.netmask = NULL;
+
+	g_free(parent->ipv4.nameserver);
+	parent->ipv4.nameserver = NULL;
+
+	g_free(parent->ipv4.gateway);
+	parent->ipv4.gateway = NULL;
+
+	parent->ipv4.method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
+}
+
+static int static_network_set_connected(
+		struct connman_network *pending_network,
+				struct connman_element *parent,
+					connman_bool_t connected)
+{
+	if (connected == TRUE) {
+		struct connman_element *element;
+
+		if (parent->ipv4.address == NULL)
+			goto failed;
+
+		if (parent->ipv4.netmask == NULL)
+			goto failed;
+
+		element = connman_element_create(NULL);
+		if (element == NULL) {
+			connman_error("Can not create connman_element");
+			return -ENOMEM;
+		}
+
+		element->type = CONNMAN_ELEMENT_TYPE_IPV4;
+		element->index = parent->index;
+
+		if (connman_element_register(element, parent) < 0) {
+			connman_element_unref(element);
+			goto failed;
+		}
+	} else
+		cleanup_ipconfig(parent);
+
+	connman_network_set_connected(pending_network, connected);
+
+	return 0;
+
+failed:
+	connman_network_set_error(pending_network,
+		CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+
+	cleanup_ipconfig(parent);
+
+	return -EINVAL;
+}
+
+static gboolean pri_context_changed(DBusConnection *connection,
+					DBusMessage *message, void *user_data)
 {
 	const char *path = dbus_message_get_path(message);
+	struct connman_element *parent;
 	const char *pending_path;
 	DBusMessageIter iter, value;
 	const char *key;
@@ -1219,14 +1627,16 @@ static void pri_context_changed(DBusConnection *connection,
 	DBG("pending_network %p, path %s", pending_network, path);
 
 	if (pending_network == NULL)
-		return;
+		return TRUE;
 
 	pending_path = connman_network_get_string(pending_network, "Path");
 	if (g_strcmp0(pending_path, path) != 0)
-		return;
+		return TRUE;
+
+	parent = connman_network_get_element(pending_network);
 
 	if (dbus_message_iter_init(message, &iter) == FALSE)
-		return;
+		return TRUE;
 
 	dbus_message_iter_get_basic(&iter, &key);
 
@@ -1234,47 +1644,44 @@ static void pri_context_changed(DBusConnection *connection,
 	dbus_message_iter_recurse(&iter, &value);
 
 	if (g_str_equal(key, "Settings") == TRUE) {
-		update_settings(&value);
+
+		update_settings(&value, parent);
 	} else if (g_str_equal(key, "Active") == TRUE) {
 		dbus_bool_t active;
 
 		dbus_message_iter_get_basic(&value, &active);
-		connman_network_set_connected(pending_network, active);
+
+		switch (parent->ipv4.method) {
+		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
+			break;
+		case CONNMAN_IPCONFIG_METHOD_FIXED:
+			connman_network_set_method(pending_network,
+						CONNMAN_IPCONFIG_METHOD_FIXED);
+
+			if (static_network_set_connected(
+					pending_network, parent, active) < 0)
+				set_network_active(pending_network, FALSE);
+			break;
+		case CONNMAN_IPCONFIG_METHOD_DHCP:
+			connman_network_set_method(pending_network,
+						CONNMAN_IPCONFIG_METHOD_DHCP);
+			connman_network_set_connected(pending_network, active);
+			break;
+		}
 
 		pending_network = NULL;
 	}
+
+	return TRUE;
 }
 
-static DBusHandlerResult ofono_signal(DBusConnection *connection,
-					DBusMessage *message, void *user_data)
-{
-	if (dbus_message_is_signal(message, OFONO_MODEM_INTERFACE,
-						PROPERTY_CHANGED) == TRUE) {
-		modem_changed(connection, message);
-	} else if (dbus_message_is_signal(message, OFONO_GPRS_INTERFACE,
-						PROPERTY_CHANGED) == TRUE) {
-		gprs_changed(connection, message);
-	} else if (dbus_message_is_signal(message, OFONO_MANAGER_INTERFACE,
-						PROPERTY_CHANGED) == TRUE) {
-		manager_changed(connection, message);
-	} else if (dbus_message_is_signal(message, OFONO_PRI_CONTEXT_INTERFACE,
-						PROPERTY_CHANGED) == TRUE) {
-		pri_context_changed(connection, message);
-	}
-
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-}
-
-static const char *gprs_rule = "type=signal, member=" PROPERTY_CHANGED
-					",interface=" OFONO_GPRS_INTERFACE;
-static const char *modem_rule = "type=signal,member=" PROPERTY_CHANGED
-					",interface=" OFONO_MODEM_INTERFACE;
-static const char *manager_rule = "type=signal,member=" PROPERTY_CHANGED
-					",interface=" OFONO_MANAGER_INTERFACE;
-static const char *pri_context_rule = "type=signal,member=" PROPERTY_CHANGED
-				", interface=" OFONO_PRI_CONTEXT_INTERFACE;
-
 static guint watch;
+static guint gprs_watch;
+static guint modem_watch;
+static guint manager_watch;
+static guint context_watch;
 
 static int ofono_init(void)
 {
@@ -1284,10 +1691,37 @@ static int ofono_init(void)
 	if (connection == NULL)
 		return -EIO;
 
-	if (dbus_connection_add_filter(connection, ofono_signal,
-						NULL, NULL) == FALSE) {
+	watch = g_dbus_add_service_watch(connection, OFONO_SERVICE,
+			ofono_connect, ofono_disconnect, NULL, NULL);
+
+	gprs_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_GPRS_INTERFACE,
+						PROPERTY_CHANGED,
+						gprs_changed,
+						NULL, NULL);
+
+	modem_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_MODEM_INTERFACE,
+						PROPERTY_CHANGED,
+						modem_changed,
+						NULL, NULL);
+
+	manager_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_MANAGER_INTERFACE,
+						PROPERTY_CHANGED,
+						manager_changed,
+						NULL, NULL);
+
+	context_watch = g_dbus_add_signal_watch(connection, NULL, NULL,
+						OFONO_PRI_CONTEXT_INTERFACE,
+						PROPERTY_CHANGED,
+						pri_context_changed,
+						NULL, NULL);
+
+	if (watch == 0 || gprs_watch == 0 || modem_watch == 0 ||
+			manager_watch == 0 || context_watch == 0) {
 		err = -EIO;
-		goto unref;
+		goto remove;
 	}
 
 	err = connman_network_driver_register(&network_driver);
@@ -1300,24 +1734,15 @@ static int ofono_init(void)
 		goto remove;
 	}
 
-	watch = g_dbus_add_service_watch(connection, OFONO_SERVICE,
-			ofono_connect, ofono_disconnect, NULL, NULL);
-	if (watch == 0) {
-		err = -EIO;
-		goto remove;
-	}
-
-	dbus_bus_add_match(connection, modem_rule, NULL);
-	dbus_bus_add_match(connection, gprs_rule, NULL);
-	dbus_bus_add_match(connection, manager_rule, NULL);
-	dbus_bus_add_match(connection, pri_context_rule, NULL);
-
 	return 0;
 
 remove:
-	dbus_connection_remove_filter(connection, ofono_signal, NULL);
+	g_dbus_remove_watch(connection, watch);
+	g_dbus_remove_watch(connection, gprs_watch);
+	g_dbus_remove_watch(connection, modem_watch);
+	g_dbus_remove_watch(connection, manager_watch);
+	g_dbus_remove_watch(connection, context_watch);
 
-unref:
 	dbus_connection_unref(connection);
 
 	return err;
@@ -1325,20 +1750,17 @@ unref:
 
 static void ofono_exit(void)
 {
-	dbus_bus_remove_match(connection, modem_rule, NULL);
-	dbus_bus_remove_match(connection, gprs_rule, NULL);
-	dbus_bus_remove_match(connection, manager_rule, NULL);
-	dbus_bus_remove_match(connection, pri_context_rule, NULL);
-
 	g_dbus_remove_watch(connection, watch);
+	g_dbus_remove_watch(connection, gprs_watch);
+	g_dbus_remove_watch(connection, modem_watch);
+	g_dbus_remove_watch(connection, manager_watch);
+	g_dbus_remove_watch(connection, context_watch);
 
 	ofono_disconnect(connection, NULL);
 
 	connman_device_driver_unregister(&modem_driver);
 	connman_network_driver_unregister(&network_driver);
 
-	dbus_connection_remove_filter(connection, ofono_signal, NULL);
-
 	dbus_connection_unref(connection);
 }
 
