@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -26,13 +26,15 @@
 #include <stdio.h>
 #include <unistd.h>
 
+#include <gdbus.h>
+
 #define CONNMAN_API_SUBJECT_TO_CHANGE
 #include <connman/plugin.h>
 #include <connman/driver.h>
+#include <connman/inet.h>
 #include <connman/dbus.h>
 #include <connman/log.h>
 
-#include "inet.h"
 #include "task.h"
 
 #define UDHCPC_INTF  "net.busybox.udhcpc"
@@ -49,7 +51,7 @@ static int udhcp_probe(struct connman_element *element)
 	if (access(UDHCPC, X_OK) < 0)
 		return -errno;
 
-	ifname = inet_index2name(element->index);
+	ifname = connman_inet_ifname(element->index);
 	if (ifname == NULL)
 		return -ENOMEM;
 
@@ -118,7 +120,7 @@ static void udhcp_bound(DBusMessage *msg, gboolean renew)
 
 	DBG("%s ==> address %s gateway %s", interface, address, gateway);
 
-	index = inet_name2index(interface);
+	index = connman_inet_ifindex(interface);
 	if (index < 0)
 		return;
 
@@ -161,25 +163,25 @@ static void udhcp_bound(DBusMessage *msg, gboolean renew)
 		connman_element_unref(element);
 }
 
-static DBusHandlerResult udhcp_filter(DBusConnection *conn,
+static gboolean udhcp_filter(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	if (dbus_message_is_method_call(msg, UDHCPC_INTF, "bound") == TRUE) {
 		udhcp_bound(msg, FALSE);
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+		return TRUE;
 	}
 
 	if (dbus_message_is_method_call(msg, UDHCPC_INTF, "renew") == TRUE) {
 		udhcp_bound(msg, TRUE);
-		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+		return TRUE;
 	}
 
-	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	return TRUE;
 }
 
 static DBusConnection *connection;
 
-static const char *udhcp_rule = "path=" UDHCPC_PATH ",interface=" UDHCPC_INTF;
+static guint watch;
 
 static int udhcp_init(void)
 {
@@ -187,9 +189,11 @@ static int udhcp_init(void)
 
 	connection = connman_dbus_get_connection();
 
-	dbus_connection_add_filter(connection, udhcp_filter, NULL, NULL);
-
-	dbus_bus_add_match(connection, udhcp_rule, NULL);
+	watch = g_dbus_add_signal_watch(connection, NULL, UDHCPC_PATH,
+					UDHCPC_INTF, NULL, udhcp_filter,
+					NULL, NULL);
+	if (watch == 0)
+		return -EIO;
 
 	err = connman_driver_register(&udhcp_driver);
 	if (err < 0) {
@@ -204,9 +208,7 @@ static void udhcp_exit(void)
 {
 	connman_driver_unregister(&udhcp_driver);
 
-	dbus_bus_remove_match(connection, udhcp_rule, NULL);
-
-	dbus_connection_remove_filter(connection, udhcp_filter, NULL);
+	g_dbus_remove_watch(connection, watch);
 
 	dbus_connection_unref(connection);
 }
